var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import React, { forwardRef, useEffect, useImperativeHandle, useState } from 'react';
import { formbuilder } from '../utils/FormBuilderInitializer';
import { useStateRef } from '../hooks/useStateRef';
import { formListHelper } from '../builders/helpers/FormListHelper';
export const FormList = forwardRef((props, ref) => {
    var _a, _b, _c, _d, _e;
    // verify that the formbuilder is set up correctly
    formbuilder.verify();
    const [listProps, setListProps] = useState(Object.assign({}, props));
    const [items, setItems, itemsRef] = useStateRef((_a = props.items) !== null && _a !== void 0 ? _a : []);
    const [filteredItems, setFilteredItems, filteredItemsRef] = useStateRef((_b = props.items) !== null && _b !== void 0 ? _b : []);
    const [selectedItems, setSelectedItems, selectedItemsRef] = useStateRef([]);
    const [columns, setColumns, columnRef] = useStateRef();
    const [enableShimmer, setEnableShimmer] = useState(false);
    useEffect(() => {
        var _a, _b, _c, _d;
        setItems((_a = props.items) !== null && _a !== void 0 ? _a : []);
        setFilteredItems((_b = props.items) !== null && _b !== void 0 ? _b : []);
        // update selected items, when changes to items from outside (ewi)
        if ((_c = selectedItemsRef.current) === null || _c === void 0 ? void 0 : _c.length) {
            const changedItems = ((_d = props.items) !== null && _d !== void 0 ? _d : []).filter(_ => selectedItemsRef.current.findIndex(r => props.listProps.config.itemIdentifier(r) == props.listProps.config.itemIdentifier(_)) >= 0);
            setSelectedItems(changedItems);
        }
    }, [props.items]);
    useImperativeHandle(ref, () => ({
        getItem: () => items
    }), [items]);
    useEffect(() => {
        let clone = Object.assign({}, props);
        let callback = clone.listProps.config.onItemsChange;
        clone.listProps.config.onItemsChange = (items) => {
            setItems(items);
            if (callback)
                callback(items);
        };
        setListProps(clone);
    }, [props.listProps, props.schema]);
    useEffect(() => {
        const cols = buildColumns();
        setColumns(cols);
    }, []);
    useEffect(() => {
        var _a, _b;
        const shimmerConfig = (_a = props.listProps) === null || _a === void 0 ? void 0 : _a.shimmerConfig;
        if (shimmerConfig === null || shimmerConfig === void 0 ? void 0 : shimmerConfig.forceShimmer) {
            setEnableShimmer(true);
        }
        else if ((shimmerConfig === null || shimmerConfig === void 0 ? void 0 : shimmerConfig.autoShimmerOnEmptyList) && !!!(items === null || items === void 0 ? void 0 : items.length)) {
            setEnableShimmer(true);
            setTimeout(() => setEnableShimmer(false), (_b = shimmerConfig === null || shimmerConfig === void 0 ? void 0 : shimmerConfig.autoShimmerTimeout) !== null && _b !== void 0 ? _b : 5000);
        }
        else {
            setEnableShimmer(false);
        }
    }, [items, (_c = props.listProps) === null || _c === void 0 ? void 0 : _c.shimmerConfig, (_e = (_d = props.listProps) === null || _d === void 0 ? void 0 : _d.shimmerConfig) === null || _e === void 0 ? void 0 : _e.forceShimmer]);
    if ((listProps === null || listProps === void 0 ? void 0 : listProps.schema) == null)
        return null;
    const itemChangeInCollection = (collection, item) => {
        let clone = [...collection];
        let itemIndex = clone.findIndex(_ => props.listProps.config.itemIdentifier(_) == props.listProps.config.itemIdentifier(item));
        if (itemIndex < 0)
            clone.unshift(item);
        else
            clone[itemIndex] = Object.assign({}, item);
        return clone;
    };
    const onItemChange = (item) => __awaiter(void 0, void 0, void 0, function* () {
        if (item == null)
            return;
        // changed
        let changedItems = itemChangeInCollection(itemsRef.current, item);
        // callbacks
        if (props.listProps.onItemChange)
            yield props.listProps.onItemChange(Object.assign({}, item));
        if (props.listProps.config.onItemsChange)
            props.listProps.config.onItemsChange([...changedItems]);
        // items
        setItems(changedItems);
        // filtered
        let changedFilteredItems = itemChangeInCollection(filteredItemsRef.current, item);
        setFilteredItems(changedFilteredItems);
        // selected
        let changedSelectedItems = itemChangeInCollection(selectedItemsRef.current, item);
        setSelectedItems(changedSelectedItems);
    });
    const columnValueRender = (propInfo) => {
        var _a;
        let valueRender = (_a = propInfo === null || propInfo === void 0 ? void 0 : propInfo.listItemOptions) === null || _a === void 0 ? void 0 : _a.customValueRender;
        if (valueRender == undefined)
            return undefined;
        else {
            return (item) => valueRender(item, ((item) => onItemChange(item)));
        }
    };
    const deleteItems = (pre) => __awaiter(void 0, void 0, void 0, function* () {
        let sItems = [...selectedItems];
        // pre event (can cancel the operation)
        if (pre) {
            let result = yield pre(sItems);
            if (result === false)
                return false;
        }
        let itemsLeft = [];
        let filteredItemsLeft = [];
        // not all items
        if (items.length != sItems.length) {
            itemsLeft = items.filter(_ => sItems.indexOf(_) < 0);
            filteredItemsLeft = filteredItems.filter(_ => sItems.indexOf(_) < 0);
        }
        setItems(itemsLeft);
        setFilteredItems(filteredItemsLeft);
        setSelectedItems([]);
        if (props.listProps.onItemsRemoved)
            props.listProps.onItemsRemoved([...sItems]);
        if (props.listProps.config.onItemsChange)
            props.listProps.config.onItemsChange([...itemsLeft]);
    });
    const buildColumns = () => {
        var _a, _b;
        let propertyOptions = props.schema.options.properties;
        let columnOptions = (_a = props.listProps.columnConfig) === null || _a === void 0 ? void 0 : _a.columnsPicks;
        let propKeys = Object.keys(propertyOptions);
        let filteredPropKeys = formListHelper.filterColumns(propKeys, columnOptions);
        let columns = filteredPropKeys.map(_ => {
            var _a, _b;
            let propInfo = propertyOptions[_];
            let col = {
                key: _,
                fieldName: _,
                name: propInfo.displayName,
                onRender: columnValueRender(propInfo),
                minWidth: (_a = propInfo.listItemOptions) === null || _a === void 0 ? void 0 : _a.minWidth,
                isResizable: (_b = propInfo.listItemOptions) === null || _b === void 0 ? void 0 : _b.isResizable
            };
            return col;
        });
        let sortOrder = (_b = props.listProps.columnConfig) === null || _b === void 0 ? void 0 : _b.columnOrder;
        if (sortOrder != null)
            return columns.sort((a, b) => {
                if (sortOrder.indexOf(a.fieldName) == -1)
                    return 1;
                if (sortOrder.indexOf(b.fieldName) == -1)
                    return -1;
                return sortOrder.indexOf(a.fieldName) - sortOrder.indexOf(b.fieldName);
            });
        else
            return columns;
    };
    const sortColumn = (column) => {
        if (column == null)
            return;
        const newColumns = columns.slice();
        const currColumn = columnRef.current.filter(currCol => column.key === currCol.key)[0];
        newColumns.forEach((newCol) => {
            if (newCol === currColumn) {
                currColumn.isSortedDescending = !currColumn.isSortedDescending;
                currColumn.isSorted = true;
            }
            else {
                newCol.isSorted = false;
                newCol.isSortedDescending = true;
            }
        });
        const property = currColumn.fieldName;
        const newItems = formListHelper.copyAndSort(filteredItemsRef.current, property, currColumn.isSortedDescending);
        setColumns(newColumns);
        setFilteredItems(newItems);
    };
    const ListContainer = formbuilder.formItemRender.list(); // WTF!! added readOnly to listprops and suddently it wont compile - works fine though (ewi)
    return (React.createElement("div", { className: "formbuilder-listcontainer", key: `${props.keyPrefix}` },
        React.createElement(ListContainer, Object.assign({}, listProps, { items: items, filteredItems: filteredItems, selectedItems: selectedItems, onItemChange: onItemChange !== null && onItemChange !== void 0 ? onItemChange : ((item) => { }), deleteItems: deleteItems, columnValueRender: columnValueRender, columns: columns !== null && columns !== void 0 ? columns : [], updateItems: setItems, updateFilteredItems: setFilteredItems, updateSelectedItems: setSelectedItems, sortColumn: sortColumn, readOnly: props.readOnly, shimmer: enableShimmer }))));
});
//# sourceMappingURL=FormList.js.map